/*
 * TerraTec Cinergy T�/qanu USB2 DVB-T adapter.
 *
 * Copyright (C) 2004 Daniel Mack <daniel@qanu.de> and
 *		    Holger Waechtler <holger@qanu.de>
 *
 *  Protocol Spec published on http://qanu.de/specs/terratec_cinergyT2.pdf
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "cinergyT2.h"


// #ifdef CONFIG_DVB_CINERGYT2_TUNING
// 	#define STREAM_URB_COUNT (CONFIG_DVB_CINERGYT2_STREAM_URB_COUNT)
// 	#define STREAM_BUF_SIZE (CONFIG_DVB_CINERGYT2_STREAM_BUF_SIZE)
// 	#define QUERY_INTERVAL (CONFIG_DVB_CINERGYT2_QUERY_INTERVAL)
// 	#ifdef CONFIG_DVB_CINERGYT2_ENABLE_RC_INPUT_DEVICE
// 		#define RC_QUERY_INTERVAL (CONFIG_DVB_CINERGYT2_RC_QUERY_INTERVAL)
// 		// #define ENABLE_RC (1)
// 	#endif
// #else
// 	#define STREAM_URB_COUNT (32)
// 	#define STREAM_BUF_SIZE (512)	/* bytes */
// 	// #define ENABLE_RC (1)
// 	#define RC_QUERY_INTERVAL (50)	/* milliseconds */
// 	#define QUERY_INTERVAL (333)	/* milliseconds */
// #endif

/* debug */
int dvb_usb_cinergyt2_debug;
module_param_named(debug,dvb_usb_cinergyt2_debug, int, 0644);
MODULE_PARM_DESC(debug, "set debugging level (1=info,xfer=2,rc=4 (or-able)).");



int cinergyt2_command(struct dvb_usb_device *d,
			      char *send_buf, int send_buf_len,
			      char *recv_buf, int recv_buf_len, int msec)
{
	int actual_len;
	char dummy;
	int ret;
	msec=msec;

	if ((ret = mutex_lock_interruptible(&d->usb_mutex)))
		return ret;

	ret = usb_bulk_msg(d->udev, usb_sndbulkpipe(d->udev, 1),
	 		   send_buf, send_buf_len, &actual_len, 1000);
	if (ret) {
		err("usb_bulk_msg (send) failed, err %i\n", ret);
		mutex_unlock(&d->usb_mutex);
		return (-EIO);
	}

	if (!recv_buf)
		recv_buf = &dummy;

	ret = usb_bulk_msg(d->udev, usb_rcvbulkpipe(d->udev, 1),
			   recv_buf, recv_buf_len, &actual_len, 1000);

	if (ret) {
		err("usb_bulk_msg (read) failed, err %i\n", ret);
		mutex_unlock(&d->usb_mutex);
		return (-EIO);
	}

	mutex_unlock(&d->usb_mutex);
	return actual_len;
}

static int cinergyt2_streaming_ctrl(struct dvb_usb_adapter *adap, int enable)
{
	char buf [] = { CINERGYT2_EP1_CONTROL_STREAM_TRANSFER, enable ? 1 : 0 };
	return cinergyt2_command(adap->dev, buf, sizeof(buf), NULL, 0, 30);
}

static int cinergyt2_power_ctrl(struct dvb_usb_device *d, int enable)
{
	char query[] = { CINERGYT2_EP1_GET_FIRMWARE_VERSION };
	char buf[] = { CINERGYT2_EP1_SLEEP_MODE, enable ? 0 : 1 };
	char state[3];
	int ret;

#if 0
	ret = cinergyt2_command(d, query, sizeof(query), state, sizeof(state), 0);
	if (ret < 0) {
		info("cinergyt2_power_ctrl() Failed to retrieve sleep state info\n");
		return ret;
	}

	if (enable == state[2]) {
		printk(KERN_INFO "cinergyt2_power_ctrl() Already in correct power state=%d\n", enable);
		return 0;
	}
#endif
	info("cinergyt2_power_ctrl() setting power=%d\n", enable);
	return cinergyt2_command(d, buf, sizeof(buf), NULL, 0, 0);
}

static int cinergyt2_frontend_attach(struct dvb_usb_adapter *adap)
{
	adap->fe = cinergyt2_fe_attach(adap->dev);
	return 0;
}



static struct dvb_usb_device_properties cinergyt2_properties;

static int cinergyt2_usb_probe(struct usb_interface *intf,
                const struct usb_device_id *id)
{
        return dvb_usb_device_init(intf,&cinergyt2_properties,THIS_MODULE,NULL);
}


static struct usb_device_id cinergyt2_usb_table [] = {
	{ USB_DEVICE(USB_VID_TERRATEC, 0x0038) },
	{ 0 }
};

MODULE_DEVICE_TABLE(usb, cinergyt2_usb_table);


static struct dvb_usb_device_properties cinergyt2_properties = {
	// .usb_ctrl = CYPRESS_FX2,
	// .firmware = "dvb-usb-vp7045-01.fw",

	.num_adapters = 1,
	.adapter = {
		{
			.streaming_ctrl   = cinergyt2_streaming_ctrl,
			.frontend_attach  = cinergyt2_frontend_attach,

			/* parameter for the MPEG2-data transfer */
			.stream = {
				.type = USB_BULK,
				.count = 7,
				.endpoint = 0x02,
				.u = {
					.bulk = {
						.buffersize = 512,
					}
				}
			},
		}
	},
	.power_ctrl       = cinergyt2_power_ctrl,

	// .rc_interval      = 300,
	// .rc_key_map       = cinergyt2_rc_keys,
	// .rc_key_map_size  = 8,
	// .rc_query         = cinergyt2_rc_query,

        .generic_bulk_ctrl_endpoint = 0x01,

	.num_device_descs = 1,
	.devices = {
		{ .name = "TerraTec/qanu USB2.0 Highspeed DVB-T Receiver",
		  .cold_ids = {NULL},
		  .warm_ids = { &cinergyt2_usb_table[0], NULL },
		},
		{ NULL },
	}
};


static struct usb_driver cinergyt2_driver = {
#if LINUX_VERSION_CODE <=  KERNEL_VERSION(2,6,15)
	.owner	= THIS_MODULE,
#endif
	.name		= "cinergyT2",
	.probe		= cinergyt2_usb_probe,
	.disconnect	= dvb_usb_device_exit,
	.id_table	= cinergyt2_usb_table
};

static int __init cinergyt2_usb_init(void)
{
	int err;
	int size;

#if 0
	size = cinergyt2_rc_keys_size();
	if (size > 0) {
		err("Cinergy Remote Control was not enabled!\n");
		cinergyt2_properties.rc_key_map_size = size;
		cinergyt2_properties.rc_key_map = cinergyt2_rc_keys;
	} else {
		cinergyt2_properties.rc_query = NULL;
		cinergyt2_properties.rc_interval = 0;
	}

	err("Cinergy Remote Control table size=%d\n", size);
#endif

	if ((err = usb_register(&cinergyt2_driver)))
		err("usb_register() failed! (err %i)\n", err);
	return err;
}

static void __exit cinergyt2_usb_exit(void)
{
	usb_deregister(&cinergyt2_driver);
}

module_init (cinergyt2_usb_init);
module_exit (cinergyt2_usb_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Holger Waechtler, Daniel Mack");


